@@ -1,12 +1,22 @@
 from __future__ import division
+
+from warnings import warn
+from math import factorial
+
 import numpy as np
+
+from scipy.special import genlaguerre, gamma, hyp2f1
+
 from dipy.reconst.cache import Cache
 from dipy.reconst.multi_voxel import multi_voxel_fit
 from dipy.reconst.shm import real_sph_harm
-from dipy.core.gradients import gradient_table
-from scipy.special import genlaguerre, gamma, hyp2f1
 from dipy.core.geometry import cart2sphere
-from math import factorial
+
+from dipy.utils.optpkg import optional_package
+
+cvxopt, have_cvxopt, _ = optional_package("cvxopt")
+if have_cvxopt:
+    import cvxopt.solvers
 
 class ShoreModel(Cache):
 
@@ -50,6 +60,13 @@ class ShoreModel(Cache):
     .. [5] Ozarslan E. et. al, "Mean apparent propagator (MAP) MRI: A novel
            diffusion imaging method for mapping tissue microstructure",
            NeuroImage, 2013.
+
+    Notes
+    -----
+    The implementation of SHORE depends on CVXOPT (http://cvxopt.org/). This
+    software is licensed under the GPL (see:
+    http://cvxopt.org/copyright.html).and you may be subject to this license
+    when using SHORE.
     """
 
     def __init__(self,
@@ -57,7 +74,13 @@ class ShoreModel(Cache):
                  radial_order=6,
                  zeta=700,
                  lambdaN=1e-8,
-                 lambdaL=1e-8):
+                 lambdaL=1e-8,
+                 tau=1. / (4 * np.pi ** 2),
+                 constrain_e0=False,
+                 positive_constraint=False,
+                 pos_grid=11,
+                 pos_radius=20e-03
+                 ):
         r""" Analytical and continuous modeling of the diffusion signal with
         respect to the SHORE basis [1,2]_.
         This implementation is a modification of SHORE presented in [1]_.
@@ -95,6 +118,16 @@ class ShoreModel(Cache):
         tau : float,
             diffusion time. By default the value that makes q equal to the
             square root of the b-value.
+        constrain_e0 : bool,
+            Constrain the optimization such that E(0) = 1.
+        positive_constraint : bool,
+            Constrain the propagator to be positive.
+        pos_grid : int,
+            Grid that define the points of the EAP in which we want to enforce
+            positivity.
+        pos_radius : float,
+            Radius of the grid of the EAP in which enforce positivity in
+            millimeters. By default 20e-03 mm.
 
         References
         ----------
@@ -124,7 +157,7 @@ class ShoreModel(Cache):
         gtab = gradient_table(bvals, bvecs)
         from dipy.sims.voxel import SticksAndBall
         data, golden_directions = SticksAndBall(gtab, d=0.0015,
-                                                S0=1, angles=[(0, 0), (90, 0)],
+                                                S0=1., angles=[(0, 0), (90, 0)],
                                                 fractions=[50, 50], snr=None)
         from dipy.reconst.canal import ShoreModel
         radial_order = 4
@@ -138,6 +171,7 @@ class ShoreModel(Cache):
         self.bvals = gtab.bvals
         self.bvecs = gtab.bvecs
         self.gtab = gtab
+        self.constrain_e0 = constrain_e0
         if radial_order > 0 and not(bool(radial_order % 2)):
             self.radial_order = radial_order
         else:
@@ -147,31 +181,104 @@ class ShoreModel(Cache):
         self.lambdaL = lambdaL
         self.lambdaN = lambdaN
         if (gtab.big_delta is None) or (gtab.small_delta is None):
-            self.tau = 1 / (4 * np.pi ** 2)
+            self.tau = tau
         else:
             self.tau = gtab.big_delta - gtab.small_delta / 3.0
 
+        if positive_constraint and not(constrain_e0):
+            msg = "Constrain_e0 must be True to enfore positivity."
+            raise ValueError(msg)
+        self.positive_constraint = positive_constraint
+        self.pos_grid = pos_grid
+        self.pos_radius = pos_radius
+
     @multi_voxel_fit
     def fit(self, data):
+
         Lshore = l_shore(self.radial_order)
         Nshore = n_shore(self.radial_order)
         # Generate the SHORE basis
         M = self.cache_get('shore_matrix', key=self.gtab)
         if M is None:
-            M = shore_matrix(self.radial_order,  self.zeta, self.gtab, self.tau)
+            M = shore_matrix(
+                self.radial_order,  self.zeta, self.gtab, self.tau)
             self.cache_set('shore_matrix', self.gtab, M)
 
+        MpseudoInv = self.cache_get('shore_matrix_reg_pinv', key=self.gtab)
+        if MpseudoInv is None:
+            MpseudoInv = np.dot(
+                np.linalg.inv(np.dot(M.T, M) + self.lambdaN * Nshore + self.lambdaL * Lshore), M.T)
+            self.cache_set('shore_matrix_reg_pinv', self.gtab, MpseudoInv)
+
         # Compute the signal coefficients in SHORE basis
-        pseudoInv = np.dot(np.linalg.inv(np.dot(M.T, M) + self.lambdaN * Nshore + self.lambdaL * Lshore), M.T)
-        coef = np.dot(pseudoInv, data)
+        if not self.constrain_e0:
+            coef = np.dot(MpseudoInv, data)
 
-        signal_0 = 0
+            signal_0 = 0
 
-        for n in range(int(self.radial_order / 2) + 1):
-            signal_0 += coef[n] * genlaguerre(n, 0.5)(0) * \
-                ((factorial(n)) / (2 * np.pi * (self.zeta ** 1.5) * gamma(n + 1.5))) ** 0.5
+            for n in range(int(self.radial_order / 2) + 1):
+                signal_0 += (
+                    coef[n] * (genlaguerre(n, 0.5)(0) * (
+                        (factorial(n)) /
+                        (2 * np.pi * (self.zeta ** 1.5) * gamma(n + 1.5))
+                    ) ** 0.5)
+                )
 
-        coef = coef / signal_0
+            coef = coef / signal_0
+        else:
+            data = data / data[self.gtab.b0s_mask].mean()
+
+            # If cvxopt is not available, bail (scipy is ~100 times slower)
+            if not have_cvxopt:
+                raise ValueError(
+                    'CVXOPT package needed to enforce constraints')
+            w_s = "The implementation of SHORE depends on CVXOPT "
+            w_s += " (http://cvxopt.org/). This software is licensed "
+            w_s += "under the GPL (see: http://cvxopt.org/copyright.html) "
+            w_s += " and you may be subject to this license when using SHORE."
+            warn(w_s)
+            M0 = M[self.gtab.b0s_mask, :]
+            M0_mean = M0.mean(0)[None, :]
+            Mprime = np.r_[M0_mean, M[~self.gtab.b0s_mask, :]]
+            Q = cvxopt.matrix(np.ascontiguousarray(
+                np.dot(Mprime.T, Mprime)
+                + self.lambdaN * Nshore + self.lambdaL * Lshore
+            ))
+
+            data_b0 = data[self.gtab.b0s_mask].mean()
+            data_single_b0 = np.r_[
+                data_b0, data[~self.gtab.b0s_mask]] / data_b0
+            p = cvxopt.matrix(np.ascontiguousarray(
+                -1 * np.dot(Mprime.T, data_single_b0))
+            )
+
+            cvxopt.solvers.options['show_progress'] = False
+
+            if not(self.positive_constraint):
+                G = None
+                h = None
+            else:
+                lg = int(np.floor(self.pos_grid ** 3 / 2))
+                G = self.cache_get(
+                    'shore_matrix_positive_constraint', key=(self.pos_grid, self.pos_radius))
+                if G is None:
+                    v, t = create_rspace(self.pos_grid, self.pos_radius)
+
+                    psi = shore_matrix_pdf(
+                        self.radial_order, self.zeta, t[:lg])
+                    G = cvxopt.matrix(-1 * psi)
+                    self.cache_set(
+                        'shore_matrix_positive_constraint', (self.pos_grid, self.pos_radius), G)
+                h = cvxopt.matrix((1e-10) * np.ones((lg)), (lg, 1))
+
+            A = cvxopt.matrix(np.ascontiguousarray(M0_mean))
+            b = cvxopt.matrix(np.array([1.]))
+            sol = cvxopt.solvers.qp(Q, p, G, h, A, b)
+
+            if sol['status'] != 'optimal':
+                warn('Optimization did not find a solution')
+
+            coef = np.array(sol['x'])[:, 0]
 
         return ShoreFit(self, coef)
 
@@ -213,31 +320,44 @@ class ShoreFit():
             the ensemble average propagator in the 3D grid
 
         """
-        eap = np.zeros((gridsize, gridsize, gridsize))
         # Create the grid in which to compute the pdf
-        rgrid, rtab = create_rspace(gridsize, radius_max)
-        psi = self.model.cache_get('shore_matrix_pdf', key=gridsize)
+        rgrid_rtab = self.model.cache_get(
+            'pdf_grid', key=(gridsize, radius_max))
+        if rgrid_rtab is None:
+            rgrid_rtab = create_rspace(gridsize, radius_max)
+            self.model.cache_set(
+                'pdf_grid', (gridsize, radius_max), rgrid_rtab)
+        rgrid, rtab = rgrid_rtab
+
+        psi = self.model.cache_get(
+            'shore_matrix_pdf', key=(gridsize, radius_max))
         if psi is None:
             psi = shore_matrix_pdf(self.radial_order,  self.zeta, rtab)
-            self.model.cache_set('shore_matrix_pdf', gridsize, psi)
+            self.model.cache_set(
+                'shore_matrix_pdf', (gridsize, radius_max), psi)
 
         propagator = np.dot(psi, self._shore_coef)
-        # fill real space
-        for i in range(len(rgrid)):
-            qx, qy, qz = rgrid[i]
-            eap[qx, qy, qz] += propagator[i]
-        # normalize by the area of the propagator
-        eap = eap * (2 * radius_max / (gridsize - 1)) ** 3
+        eap = np.empty((gridsize, gridsize, gridsize), dtype=float)
+        eap[tuple(rgrid.astype(int).T)] = propagator
+        eap *= (2 * radius_max / (gridsize - 1)) ** 3
 
-        return np.clip(eap, 0, eap.max())
+        return eap
 
     def pdf(self, r_points):
         """ Diffusion propagator on a given set of real points.
+            if the array r_points is non writeable, then intermediate
+            results are cached for faster recalculation
         """
-        psi = self.model.cache_get('shore_matrix_pdf', key=r_points.sum())
+        if not r_points.flags.writeable:
+            psi = self.model.cache_get(
+                'shore_matrix_pdf', key=hash(r_points.data))
+        else:
+            psi = None
         if psi is None:
             psi = shore_matrix_pdf(self.radial_order,  self.zeta, r_points)
-            self.model.cache_set('shore_matrix_pdf', r_points.sum(), psi)
+            if not r_points.flags.writeable:
+                self.model.cache_set(
+                    'shore_matrix_pdf', hash(r_points.data), psi)
 
         eap = np.dot(psi, self._shore_coef)
 
@@ -247,7 +367,7 @@ class ShoreFit():
         r""" Calculates the real analytical ODF in terms of Spherical Harmonics.
         """
         # Number of Spherical Harmonics involved in the estimation
-        J = (self.radial_order + 1) * (self.radial_order + 2) / 2
+        J = (self.radial_order + 1) * (self.radial_order + 2) // 2
 
         # Compute the Spherical Harmonics Coefficients
         c_sh = np.zeros(J)
@@ -347,6 +467,12 @@ class ShoreFit():
 
         return np.clip(msd, 0, msd.max())
 
+    def fitted_signal(self):
+        """ The fitted signal.
+        """
+        phi = self.model.cache_get('shore_matrix', key=self.model.gtab)
+        return np.dot(phi, self._shore_coef)
+
     @property
     def shore_coeff(self):
         """The SHORE coefficients
@@ -398,6 +524,7 @@ def shore_matrix(radial_order, zeta, gtab, tau=1 / (4 * np.pi ** 2)):
     """
 
     qvals = np.sqrt(gtab.bvals / (4 * np.pi ** 2 * tau))
+    qvals[gtab.b0s_mask] = 0
     bvecs = gtab.bvecs
 
     qgradients = qvals[:, None] * bvecs
@@ -406,7 +533,7 @@ def shore_matrix(radial_order, zeta, gtab, tau=1 / (4 * np.pi ** 2)):
                                 qgradients[:, 2])
     theta[np.isnan(theta)] = 0
     F = radial_order / 2
-    n_c = np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3))
+    n_c = int(np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3)))
     M = np.zeros((r.shape[0], n_c))
 
     counter = 0
@@ -448,7 +575,7 @@ def shore_matrix_pdf(radial_order, zeta, rtab):
     r, theta, phi = cart2sphere(rtab[:, 0], rtab[:, 1], rtab[:, 2])
     theta[np.isnan(theta)] = 0
     F = radial_order / 2
-    n_c = np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3))
+    n_c = int(np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3)))
     psi = np.zeros((r.shape[0], n_c))
     counter = 0
     for l in range(0, radial_order + 1, 2):
@@ -491,7 +618,7 @@ def shore_matrix_odf(radial_order, zeta, sphere_vertices):
                                 sphere_vertices[:, 2])
     theta[np.isnan(theta)] = 0
     F = radial_order / 2
-    n_c = np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3))
+    n_c = int(np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3)))
     upsilon = np.zeros((len(sphere_vertices), n_c))
     counter = 0
     for l in range(0, radial_order + 1, 2):
@@ -513,7 +640,7 @@ def _kappa_odf(zeta, n, l):
 def l_shore(radial_order):
     "Returns the angular regularisation matrix for SHORE basis"
     F = radial_order / 2
-    n_c = np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3))
+    n_c = int(np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3)))
     diagL = np.zeros(n_c)
     counter = 0
     for l in range(0, radial_order + 1, 2):
@@ -528,7 +655,7 @@ def l_shore(radial_order):
 def n_shore(radial_order):
     "Returns the angular regularisation matrix for SHORE basis"
     F = radial_order / 2
-    n_c = np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3))
+    n_c = int(np.round(1 / 6.0 * (F + 1) * (F + 2) * (4 * F + 3)))
     diagN = np.zeros(n_c)
     counter = 0
     for l in range(0, radial_order + 1, 2):
@@ -574,6 +701,7 @@ def create_rspace(gridsize, radius_max):
 
     return vecs, tab
 
+
 def shore_indices(radial_order, index):
     r"""Given the basis order and the index, return the shore indices n, l, m
     for modified Merlet's 3D-SHORE
@@ -636,6 +764,7 @@ def shore_indices(radial_order, index):
                     counter += 1
     return n_i, l_i, m_i
 
+
 def shore_order(n, l, m):
     r"""Given the indices (n,l,m) of the basis, return the minimum order
     for those indices and their index for modified Merlet's 3D-SHORE.
@@ -657,7 +786,7 @@ def shore_order(n, l, m):
         index of the coefficient correspondig to (n,l,m), start from 0
 
     """
-    if l % 2 == 1 or l > n or l < 0 or n < 0  or np.abs(m) > l:
+    if l % 2 == 1 or l > n or l < 0 or n < 0 or np.abs(m) > l:
         msg = "The index l must be even and 0 <= l <= n, the index m must be -l <= m <= l."
         raise ValueError(msg)
     else:
@@ -666,7 +795,7 @@ def shore_order(n, l, m):
         else:
             radial_order = n
 
-        counter_i  = 0
+        counter_i = 0
 
         counter = 0
         for l_i in range(0, radial_order + 1, 2):
