@@ -1,8 +1,9 @@
 from __future__ import division, print_function, absolute_import
 
 import numpy as np
+from nibabel.affines import apply_affine
 
-from dipy.viz.colormap import colormap_lookup_table
+from dipy.viz.colormap import colormap_lookup_table, create_colormap
 from dipy.viz.utils import lines_to_vtk_polydata
 from dipy.viz.utils import set_input
 
@@ -176,6 +177,12 @@ def slicer(data, affine=None, value_range=None, opacity=1.,
             im_actor.input_connection(self.output)
             im_actor.SetDisplayExtent(*self.GetDisplayExtent())
             im_actor.opacity(opacity)
+            if interpolation == 'nearest':
+                im_actor.SetInterpolate(False)
+            else:
+                im_actor.SetInterpolate(True)
+            if major_version >= 6:
+                im_actor.GetMapper().BorderOn()
             return im_actor
 
     image_actor = ImageActor()
@@ -276,7 +283,7 @@ def streamtube(lines, colors=None, opacity=1, linewidth=0.1, tube_sides=9,
 
     See Also
     --------
-    :func:``dipy.viz.actor.line``
+    :func:`dipy.viz.actor.line`
     """
     # Poly data with lines and colors
     poly_data, is_colormap = lines_to_vtk_polydata(lines, colors)
@@ -539,3 +546,319 @@ def axes(scale=(1, 1, 1), colorx=(1, 0, 0), colory=(0, 1, 0), colorz=(0, 0, 1),
     ass.AddPart(arrowz)
 
     return ass
+
+
+def odf_slicer(odfs, affine=None, mask=None, sphere=None, scale=2.2,
+               norm=True, radial_scale=True, opacity=1.,
+               colormap='plasma', global_cm=False):
+    """ Slice spherical fields in native or world coordinates
+
+    Parameters
+    ----------
+    odfs : ndarray
+        4D array of spherical functions
+    affine : array
+        4x4 transformation array from native coordinates to world coordinates
+    mask : ndarray
+        3D mask
+    sphere : Sphere
+        a sphere
+    scale : float
+        Distance between spheres.
+    norm : bool
+        Normalize `sphere_values`.
+    radial_scale : bool
+        Scale sphere points according to odf values.
+    opacity : float
+        Takes values from 0 (fully transparent) to 1 (opaque)
+    colormap : None or str
+        If None then white color is used. Otherwise the name of colormap is
+        given. Matplotlib colormaps are supported (e.g., 'inferno').
+    global_cm : bool
+        If True the colormap will be applied in all ODFs. If False
+        it will be applied individually at each voxel (default False).
+    """
+
+    if mask is None:
+        mask = np.ones(odfs.shape[:3], dtype=np.bool)
+    else:
+        mask = mask.astype(np.bool)
+
+    szx, szy, szz = odfs.shape[:3]
+
+    class OdfSlicerActor(vtk.vtkLODActor):
+
+        def display_extent(self, x1, x2, y1, y2, z1, z2):
+            tmp_mask = np.zeros(odfs.shape[:3], dtype=np.bool)
+            tmp_mask[x1:x2 + 1, y1:y2 + 1, z1:z2 + 1] = True
+            tmp_mask = np.bitwise_and(tmp_mask, mask)
+
+            self.mapper = _odf_slicer_mapper(odfs=odfs,
+                                             affine=affine,
+                                             mask=tmp_mask,
+                                             sphere=sphere,
+                                             scale=scale,
+                                             norm=norm,
+                                             radial_scale=radial_scale,
+                                             opacity=opacity,
+                                             colormap=colormap,
+                                             global_cm=global_cm)
+            self.SetMapper(self.mapper)
+
+        def display(self, x=None, y=None, z=None):
+            if x is None and y is None and z is None:
+                self.display_extent(0, szx - 1, 0, szy - 1,
+                                    int(np.floor(szz/2)), int(np.floor(szz/2)))
+            if x is not None:
+                self.display_extent(x, x, 0, szy - 1, 0, szz - 1)
+            if y is not None:
+                self.display_extent(0, szx - 1, y, y, 0, szz - 1)
+            if z is not None:
+                self.display_extent(0, szx - 1, 0, szy - 1, z, z)
+
+    odf_actor = OdfSlicerActor()
+    odf_actor.display_extent(0, szx - 1, 0, szy - 1,
+                             int(np.floor(szz/2)), int(np.floor(szz/2)))
+
+    return odf_actor
+
+
+def _odf_slicer_mapper(odfs, affine=None, mask=None, sphere=None, scale=2.2,
+                       norm=True, radial_scale=True, opacity=1.,
+                       colormap='plasma', global_cm=False):
+    """ Helper function for slicing spherical fields
+
+    Parameters
+    ----------
+    odfs : ndarray
+        4D array of spherical functions
+    affine : array
+        4x4 transformation array from native coordinates to world coordinates
+    mask : ndarray
+        3D mask
+    sphere : Sphere
+        a sphere
+    scale : float
+        Distance between spheres.
+    norm : bool
+        Normalize `sphere_values`.
+    radial_scale : bool
+        Scale sphere points according to odf values.
+    opacity : float
+        Takes values from 0 (fully transparent) to 1 (opaque)
+    colormap : None or str
+        If None then white color is used. Otherwise the name of colormap is
+        given. Matplotlib colormaps are supported (e.g., 'inferno').
+    global_cm : bool
+        If True the colormap will be applied in all ODFs. If False
+        it will be applied individually at each voxel (default False).
+    """
+    if mask is None:
+        mask = np.ones(odfs.shape[:3])
+
+    ijk = np.ascontiguousarray(np.array(np.nonzero(mask)).T)
+
+    if len(ijk) == 0:
+        return None
+
+    if affine is not None:
+        ijk = np.ascontiguousarray(apply_affine(affine, ijk))
+
+    faces = np.asarray(sphere.faces, dtype=int)
+    vertices = sphere.vertices
+
+    all_xyz = []
+    all_faces = []
+    all_ms = []
+    for (k, center) in enumerate(ijk):
+
+        m = odfs[tuple(center.astype(np.int))].copy()
+
+        if norm:
+            m /= np.abs(m).max()
+
+        if radial_scale:
+            xyz = vertices * m[:, None]
+        else:
+            xyz = vertices.copy()
+
+        all_xyz.append(scale * xyz + center)
+        all_faces.append(faces + k * xyz.shape[0])
+        all_ms.append(m)
+
+    all_xyz = np.ascontiguousarray(np.concatenate(all_xyz))
+    all_xyz_vtk = numpy_support.numpy_to_vtk(all_xyz, deep=True)
+
+    all_faces = np.concatenate(all_faces)
+    all_faces = np.hstack((3 * np.ones((len(all_faces), 1)),
+                           all_faces))
+    ncells = len(all_faces)
+
+    all_faces = np.ascontiguousarray(all_faces.ravel(), dtype='i8')
+    all_faces_vtk = numpy_support.numpy_to_vtkIdTypeArray(all_faces,
+                                                          deep=True)
+    if global_cm:
+        all_ms = np.ascontiguousarray(
+            np.concatenate(all_ms), dtype='f4')
+
+    points = vtk.vtkPoints()
+    points.SetData(all_xyz_vtk)
+
+    cells = vtk.vtkCellArray()
+    cells.SetCells(ncells, all_faces_vtk)
+
+    if colormap is not None:
+        if global_cm:
+            cols = create_colormap(all_ms.ravel(), colormap)
+        else:
+            cols = np.zeros((ijk.shape[0],) + sphere.vertices.shape,
+                            dtype='f4')
+            for k in range(ijk.shape[0]):
+                tmp = create_colormap(all_ms[k].ravel(), colormap)
+                cols[k] = tmp.copy()
+
+            cols = np.ascontiguousarray(
+                np.reshape(cols, (cols.shape[0] * cols.shape[1],
+                           cols.shape[2])), dtype='f4')
+
+        vtk_colors = numpy_support.numpy_to_vtk(
+            np.asarray(255 * cols),
+            deep=True,
+            array_type=vtk.VTK_UNSIGNED_CHAR)
+
+        vtk_colors.SetName("Colors")
+
+    polydata = vtk.vtkPolyData()
+    polydata.SetPoints(points)
+    polydata.SetPolys(cells)
+
+    if colormap is not None:
+        polydata.GetPointData().SetScalars(vtk_colors)
+
+    mapper = vtk.vtkPolyDataMapper()
+    if major_version <= 5:
+        mapper.SetInput(polydata)
+    else:
+        mapper.SetInputData(polydata)
+
+    return mapper
+
+
+def _makeNd(array, ndim):
+    """Pads as many 1s at the beginning of array's shape as are need to give
+    array ndim dimensions."""
+    new_shape = (1,) * (ndim - array.ndim) + array.shape
+    return array.reshape(new_shape)
+
+
+def peak_slicer(peaks_dirs, peaks_values=None, mask=None, affine=None,
+                colors=(1, 0, 0), opacity=1, linewidth=1,
+                lod=False, lod_points=10 ** 4, lod_points_size=3):
+    """ Visualize peak directions as given from ``peaks_from_model``
+
+    Parameters
+    ----------
+    peaks_dirs : ndarray
+        Peak directions. The shape of the array can be (M, 3) or (X, M, 3) or
+        (X, Y, M, 3) or (X, Y, Z, M, 3)
+    peaks_values : ndarray
+        Peak values. The shape of the array can be (M, ) or (X, M) or
+        (X, Y, M) or (X, Y, Z, M)
+
+    colors : tuple or None
+        Default red color. If None then every peak gets an orientation color
+        in similarity to a DEC map.
+
+    opacity : float, optional
+        Default is 1.
+
+    linewidth : float, optional
+        Line thickness. Default is 1.
+
+    lod : bool
+        Use vtkLODActor(level of detail) rather than vtkActor.
+        Default is False. Level of detail actors do not render the full
+        geometry when the frame rate is low.
+    lod_points : int
+        Number of points to be used when LOD is in effect. Default is 10000.
+    lod_points_size : int
+        Size of points when lod is in effect. Default is 3.
+
+    Returns
+    -------
+    vtkActor
+
+    See Also
+    --------
+    dipy.viz.fvtk.sphere_funcs
+
+    """
+    peaks_dirs = np.asarray(peaks_dirs)
+    if peaks_dirs.ndim > 5:
+        raise ValueError("Wrong shape")
+
+    peaks_dirs = _makeNd(peaks_dirs, 5)
+    if peaks_values is not None:
+        peaks_values = _makeNd(peaks_values, 4)
+
+    grid_shape = np.array(peaks_dirs.shape[:3])
+
+    if mask is None:
+        mask = np.ones(grid_shape).astype(np.bool)
+
+    class PeakSlicerActor(vtk.vtkLODActor):
+
+        def display_extent(self, x1, x2, y1, y2, z1, z2):
+
+            tmp_mask = np.zeros(grid_shape, dtype=np.bool)
+            tmp_mask[x1:x2 + 1, y1:y2 + 1, z1:z2 + 1] = True
+            tmp_mask = np.bitwise_and(tmp_mask, mask)
+
+            ijk = np.ascontiguousarray(np.array(np.nonzero(tmp_mask)).T)
+            if len(ijk) == 0:
+                self.SetMapper(None)
+                return
+            if affine is not None:
+                ijk_trans = np.ascontiguousarray(apply_affine(affine, ijk))
+            list_dirs = []
+            for index, center in enumerate(ijk):
+                # center = tuple(center)
+                if affine is None:
+                    xyz = center[:, None]
+                else:
+                    xyz = ijk_trans[index][:, None]
+                xyz = xyz.T
+                for i in range(peaks_dirs[tuple(center)].shape[-2]):
+
+                    if peaks_values is not None:
+                        pv = peaks_values[tuple(center)][i]
+                    else:
+                        pv = 1.
+                    symm = np.vstack((-peaks_dirs[tuple(center)][i] * pv + xyz,
+                                      peaks_dirs[tuple(center)][i] * pv + xyz))
+                    list_dirs.append(symm)
+
+            self.mapper = line(list_dirs, colors=colors,
+                               opacity=opacity, linewidth=linewidth,
+                               lod=lod, lod_points=lod_points,
+                               lod_points_size=lod_points_size).GetMapper()
+            self.SetMapper(self.mapper)
+
+        def display(self, x=None, y=None, z=None):
+            if x is None and y is None and z is None:
+                self.display_extent(0, szx - 1, 0, szy - 1,
+                                    int(np.floor(szz/2)), int(np.floor(szz/2)))
+            if x is not None:
+                self.display_extent(x, x, 0, szy - 1, 0, szz - 1)
+            if y is not None:
+                self.display_extent(0, szx - 1, y, y, 0, szz - 1)
+            if z is not None:
+                self.display_extent(0, szx - 1, 0, szy - 1, z, z)
+
+    peak_actor = PeakSlicerActor()
+
+    szx, szy, szz = grid_shape
+    peak_actor.display_extent(0, szx - 1, 0, szy - 1,
+                              int(np.floor(szz / 2)), int(np.floor(szz / 2)))
+
+    return peak_actor
\ No newline at end of file
