@@ -291,7 +291,7 @@ class SetupDependency(object):
         # Using setuptools; add packages to given section of
         # setup/install_requires, unless it's a heavy dependency for which we
         # already have an acceptable importable version.
-        if self.heavy and ver_err_msg:
+        if self.heavy and ver_err_msg is None:
             return
         new_req = '{0}>={1}'.format(self.import_name, self.min_ver)
         old_reqs = setuptools_kwargs.get(self.req_type, [])
@@ -325,3 +325,36 @@ def read_vars_from(ver_file):
     with open(ver_file, 'rt') as fobj:
         exec(fobj.read(), ns)
     return Bunch(ns)
+
+
+def make_np_ext_builder(build_ext_class):
+    """ Override input `build_ext_class` to add numpy includes to extension
+
+    This is useful to delay call of ``np.get_include`` until the extension is
+    being built.
+
+    Parameters
+    ----------
+    build_ext_class : class
+        Class implementing ``distutils.command.build_ext.build_ext`` interface,
+        with a ``build_extensions`` method.
+
+    Returns
+    -------
+    np_build_ext_class : class
+        A class with similar interface to
+        ``distutils.command.build_ext.build_ext``, that adds libraries in
+        ``np.get_include()`` to include directories of extension.
+    """
+    class NpExtBuilder(build_ext_class):
+
+        def build_extensions(self):
+            """ Hook into extension building to add np include dirs
+            """
+            # Delay numpy import until last moment
+            import numpy as np
+            for ext in self.extensions:
+                ext.include_dirs.append(np.get_include())
+            build_ext_class.build_extensions(self)
+
+    return NpExtBuilder
